import ssl
import smtplib

from urllib.parse import urlparse

from argos.logging import logger
from argos.schemas.config import Config, Mail

# XXX Implement mail alerts https://framagit.org/framasoft/framaspace/argos/-/issues/15
# XXX Implement gotify alerts https://framagit.org/framasoft/framaspace/argos/-/issues/16

def handle_alert(config: Config, result, task, severity):
    """Dispatch alert through configured alert channels"""
    msg = f"task={task.id}, status={result.status}, {severity=}"
    logger.error("Alerting stub: %s", msg)

    if config.general.mail is not None:
        logger.debug('Will send mail notification')

        subject = f"[Argos] {urlparse(task.url).netloc}: status {severity}"
        msg = f"""\
URL:    {task.url}
Check:  {task.check}
Status: {severity}
Time:   {result.submitted_at}
"""
        notify_by_mail(subject, msg, config.general.mail)


def notify_by_mail(subject: str, msg: str, config: Mail) -> None:
    mail = f"""\
Subject: {subject}

{msg}"""

    if config.ssl:
        logger.debug('Mail notification: SSL')
        context = ssl.create_default_context()
        smtp = smtplib.SMTP_SSL(host=config.host,
                                port=config.port,
                                context=context)
    else:
        smtp = smtplib.SMTP(host=config.host,  # type: ignore
                            port=config.port)
        if config.starttls:
            logger.debug('Mail notification: STARTTLS')
            context = ssl.create_default_context()
            smtp.starttls(context=context)

    if config.auth is not None:
        logger.debug('Mail notification: authentification')
        smtp.login(config.auth.login,
                   config.auth.password)

    for address in config.addresses:
        logger.debug('Sending mail to %s', address)
        logger.debug(msg)
        smtp.sendmail(config.mailfrom, address, mail)

    return None
